"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_js_1 = require("@solana/web3.js");
const client_1 = require("./client");
const ids_json_1 = __importDefault(require("./ids.json"));
const config_1 = require("./config");
const layout_1 = require("./layout");
const fixednum_1 = require("./fixednum");
const config = new config_1.Config(ids_json_1.default);
const cluster = (process.env.CLUSTER || 'devnet');
const groupName = process.env.GROUP || 'devnet.1';
const groupIds = config.getGroup(cluster, groupName);
if (!groupIds) {
    throw new Error(`Group ${groupName} not found`);
}
const mangoProgramId = groupIds.mangoProgramId;
const mangoGroupKey = groupIds.publicKey;
const connection = new web3_js_1.Connection(config.cluster_urls[cluster], 'processed');
const client = new client_1.MangoClient(connection, mangoProgramId);
const setUp = () => __awaiter(void 0, void 0, void 0, function* () {
    const mangoGroup = yield client.getMangoGroup(mangoGroupKey);
    const mangoAccounts = yield client.getAllMangoAccounts(mangoGroup, undefined, true);
    const perpMarkets = [];
    for (let i = 0; i < layout_1.QUOTE_INDEX; i++) {
        const perpMarketInfo = mangoGroup.perpMarkets[i];
        const perpMarket = yield client.getPerpMarket(perpMarketInfo.perpMarket, mangoGroup.tokens[i].decimals, mangoGroup.tokens[layout_1.QUOTE_INDEX].decimals);
        perpMarkets.push(perpMarket);
    }
    return { mangoGroup, mangoAccounts, perpMarkets };
});
const checkSumOfBasePositions = (mangoAccounts) => __awaiter(void 0, void 0, void 0, function* () {
    const sumOfAllBasePositions = mangoAccounts.reduce((sumAll, mangoAccount) => {
        const sumOfBasePositions = mangoAccount.perpAccounts.reduce((sum, perpAccount) => {
            return sum + perpAccount.basePosition.toNumber();
        }, 0);
        return sumAll + sumOfBasePositions;
    }, 0);
    console.log('checkSumOfBasePositions', sumOfAllBasePositions);
});
const checkSumOfQuotePositions = (mangoGroup, mangoAccounts, perpMarkets) => __awaiter(void 0, void 0, void 0, function* () {
    const mangoCache = yield mangoGroup.loadCache(connection);
    const sumOfAllQuotePositions = mangoAccounts.reduce((sumAll, mangoAccount) => {
        const sumOfQuotePositions = mangoAccount.perpAccounts.reduce((sum, perpAccount, index) => {
            const perpMarketCache = mangoCache.perpMarketCache[index];
            return sum.add(perpAccount.getQuotePosition(perpMarketCache));
        }, fixednum_1.ZERO_I80F48);
        return sumAll.add(sumOfQuotePositions);
    }, fixednum_1.ZERO_I80F48);
    const sumOfFeesAccrued = perpMarkets.reduce((sum, perpMarket) => {
        return sum.add(perpMarket.feesAccrued);
    }, fixednum_1.ZERO_I80F48);
    console.log('checkSumOfQuotePositions:', sumOfAllQuotePositions.add(sumOfFeesAccrued).toString());
});
const main = () => __awaiter(void 0, void 0, void 0, function* () {
    const { mangoGroup, mangoAccounts, perpMarkets } = yield setUp();
    yield checkSumOfBasePositions(mangoAccounts);
    yield checkSumOfQuotePositions(mangoGroup, mangoAccounts, perpMarkets);
});
main();
//# sourceMappingURL=sanityChecks.js.map